/*******************************************************
 * Copyright (C) HRider in motion S.L.
 * 
 * This file is part of HRider.
 *   
 * HRider can not be copied and/or distributed without the express
 * permission of HRider in motion S.L.
 *******************************************************/
package net.hrider.api.client;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Invocation.Builder;
import javax.ws.rs.client.WebTarget;
import org.glassfish.jersey.client.ClientProperties;
import org.glassfish.jersey.media.multipart.MultiPartFeature;

/**
 * The Web Service Api client. Modify the file <i>/src/main/resources/config.properties</i>
 * to configure the client.
 * 
 * @author drh (dani@hrider.net)
 */
public class ApiClient {
    
    private Client client;
    private Properties config;
    
    private WebTarget apiTarget;
    
    public static final String PROPERTY_API_URI = "api.uri";
    public static final String PROPERTY_API_TOKEN = "api.token";
        
    private static final Logger LOG = Logger.getLogger(ApiClient.class.getName());
    
    public ApiClient() {        
        init();
    }
    
    public ApiClient(Properties config) {
        this.config = config;        
    }
    
    public void close() {
        if (client != null) {
            client.close();
        }
    }
    
    public WebTarget getApiTarget() {
        if (apiTarget == null) {
            apiTarget = getClient().target(getBaseURI());
        }
        return apiTarget;
    }
    
    public Builder request(String type, String... path) {
        return request(type, null, path);
    }
        
    public Builder request(String type, String[] queryParams, String... path) {
        LOG.log(Level.FINE, "request ... {0}", type);
        WebTarget target = getApiTarget();
        for (String p : path) {
            LOG.log(Level.FINE, "path: {0}", p);
            target = target.path(p);
        }       
        if (queryParams != null) {
            for (int i = 0; i < queryParams.length; i = i + 2) {
                target = target.queryParam(queryParams[i], queryParams[i + 1]);
            }
        }
        Builder builder = target.request(type);
        return addHeaderAuth(builder);
    }
    
    public Builder addHeaderAuth(Builder builder) {
        builder.header("authorization", getProperty(PROPERTY_API_TOKEN));
        return builder;
    }
    
    public Client getClient() {
        if (client == null) {
            client = ClientBuilder.newBuilder().register(MultiPartFeature.class).
                    hostnameVerifier(getHostnameVerifier()).
                    sslContext(getSSLContext()).build(). // Uncomment this line for security !!!
                    property(ClientProperties.FOLLOW_REDIRECTS, Boolean.TRUE).
                    // Allow DELETES with uri list
                    property(ClientProperties.SUPPRESS_HTTP_COMPLIANCE_VALIDATION, true);
        }
        return client;
    }
    
    public String getBaseURI() {
        return getProperty(PROPERTY_API_URI);
    }
    
    public String getProperty(String key) {
        return config.getProperty(key);
    }
    
    // private methods *********************************************************
    
    private void init() {
        // load properties
        try (InputStream in = Thread.currentThread().getContextClassLoader().getResourceAsStream("config.properties")) {
            config = new Properties();
            config.load(in);
        } catch (IOException ex) { 
            throw new IllegalStateException("Invalid config properties file 'config.properties' or not found");
        }
    }
 
    private HostnameVerifier getHostnameVerifier() {
        return (String hostname, javax.net.ssl.SSLSession sslSession) -> true;
    }

    private SSLContext getSSLContext() {
        // for alternative implementation checkout org.glassfish.jersey.SslConfigurator
        javax.net.ssl.TrustManager x509 = new javax.net.ssl.X509TrustManager() {
            @Override
            public void checkClientTrusted(java.security.cert.X509Certificate[] arg0, String arg1) throws java.security.cert.CertificateException {
            }

            @Override
            public void checkServerTrusted(java.security.cert.X509Certificate[] arg0, String arg1) throws java.security.cert.CertificateException {
            }

            @Override
            public java.security.cert.X509Certificate[] getAcceptedIssuers() {
                return null;
            }
        };
        SSLContext ctx = null;
        try {
            ctx = SSLContext.getInstance("SSL");
            ctx.init(null, new javax.net.ssl.TrustManager[]{x509}, null);
        } catch (java.security.GeneralSecurityException ex) {
        }
        return ctx;
    }
    
}
